﻿local KUI, T, E, L, V, P, G = unpack(select(2, ...))
local Pr = KUI:GetModule("Professions")

local TRADE_SKILLS, NONE = TRADE_SKILLS, NONE

local function ProfessionsTable()
	E.Options.args.KlixUI.args.modules.args.professions = {
		type = "group",
		name = TRADE_SKILLS,
		order = 22,
		childGroups = 'tab',
		disabled = function() return T.IsAddOnLoaded("ElvUI_SLE") end,
		hidden = function() return T.IsAddOnLoaded("ElvUI_SLE") end,
		args = {
			name = {
				order = 1,
				type = "header",
				name = KUI:cOption(TRADE_SKILLS),
			},
			tabs = {
				order = 2,
				name = L["Profession Tabs"],
				desc = L["Creates tabs next to the profession window, similar to the spellbook tabs, for easy access to all your profession."],
				type = "toggle",
				get = function(info) return E.db.KlixUI.professions.tabs end,
				set = function(info, value) E.db.KlixUI.professions.tabs = value; E:StaticPopup_Show("PRIVATE_RL") end,
			},
			deconstructButton = {
				order = 3,
				type = "group",
				name = L["Deconstruct Mode"],
				args = {
					enable = {
						order = 1,
						name = L["Enable"],
						desc = L["Create a button in your bag frame to switch to deconstruction mode allowing you to easily disenchant/mill/prospect and pick locks."],
						type = "toggle",
						get = function(info) return E.private.KlixUI.professions.deconButton.enable end,
						set = function(info, value) E.private.KlixUI.professions.deconButton.enable = value; E:StaticPopup_Show("PRIVATE_RL") end,
					},
					style = {
						order = 2,
						type = "select",
						name = L["Style"],
						desc = L["Sets style of glow around item available for deconstruction while in deconstruct mode. Autocast is less intense but also less noticeable."],
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.private.KlixUI.professions.deconButton.style end,
						set = function(info, value) E.private.KlixUI.professions.deconButton.style = value; end,
						values = {
							["BIG"] = L["Actionbar Proc"],
							["SMALL"] = L["Actionbar Autocast"],
							["PIXEL"] = L["Pixel"],
							["NO"] = NONE,
						},
					},
					buttonGlow = {
						order = 3,
						name = L["Show Glow On Bag Button"],
						desc = L["Show glow on the deconstruction button in bag when deconstruction mode is enabled.\nApplies on next mode toggle."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.private.KlixUI.professions.deconButton.buttonGlow end,
						set = function(info, value) E.private.KlixUI.professions.deconButton.buttonGlow = value; end,
					},
				},
			},
			ench = {
				order = 4,
				type = "group",
				name = T.GetSpell(158716),
				args = {
					enchScroll = {
						order = 1,
						name = L["Enchant Scroll Button"],
						desc = L["Create a button for applying selected enchant on the scroll."],
						type = "toggle",
						get = function(info) return E.private.KlixUI.professions.enchant.enchScroll end,
						set = function(info, value) E.private.KlixUI.professions.enchant.enchScroll = value; E:StaticPopup_Show("PRIVATE_RL") end,
					},
					infos = {
						order = 2,
						type = "description",
						name = L["PROF_DESC_GLOBAL"],
					},
					ignoreItems = {
						order = 3,
						name = L["Deconstruction Ignore"],
						desc = L["Items listed here will be ignored in deconstruction mode. Add names or item links, entries must be separated by comma."],
						type = 'input',
						width = 'full',
						multiline = true,
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.global.KlixUI.DE.Blacklist end,
						set = function(info, value) E.global.KlixUI.DE.Blacklist = value; Pr:Blacklisting("DE") end,
					},
					IgnoreTabards = {
						order = 4,
						name = L["Ignore Tabards"],
						desc = L["Deconstruction mode will ignore tabards."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.global.KlixUI.DE.IgnoreTabards end,
						set = function(info, value) E.global.KlixUI.DE.IgnoreTabards = value; end,
					},
					IgnorePanda = {
						order = 5,
						name = L["Ignore Pandaria BoA"],
						desc = L["Deconstruction mode will ignore BoA weapons from Pandaria."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.global.KlixUI.DE.IgnorePanda end,
						set = function(info, value) E.global.KlixUI.DE.IgnorePanda = value; end,
					},
					IgnoreCooking = {
						order = 6,
						name = L["Ignore Cooking"],
						desc = L["Deconstruction mode will ignore cooking specific items."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.global.KlixUI.DE.IgnoreCooking end,
						set = function(info, value) E.global.KlixUI.DE.IgnoreCooking = value; end,
					},
					IgnoreFishing = {
						order = 7,
						name = L["Ignore Fishing"],
						desc = L["Deconstruction mode will ignore fishing specific items."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.global.KlixUI.DE.IgnoreFishing end,
						set = function(info, value) E.global.KlixUI.DE.IgnoreFishing = value; end,
					},
				},
			},
			lockpick = {
				order = 5,
				type = "group",
				name = T.GetSpell(1804),
				args = {
					infos = {
						order = 1,
						type = "description",
						name = L["PROF_DESC_GLOBAL"],
					},
					ignoreItems = {
						order = 2,
						name = L["Deconstruction Ignore"],
						desc = L["Items listed here will be ingored in deconstruction mode. Add names or item links, entries must be separated by comma."],
						type = 'input',
						width = 'full',
						multiline = true,
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.global.KlixUI.LOCK.Blacklist end,
						set = function(info, value) E.global.KlixUI.LOCK.Blacklist = value; Pr:Blacklisting("LOCK") end,
					},
					TradeOpen = {
						order = 3,
						name = L["Unlock In Trade"],
						desc = L["Apply unlocking skills in trade window the same way as in deconstruction mode for bags."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.deconButton.enable end,
						get = function(info) return E.global.KlixUI.LOCK.TradeOpen end,
						set = function(info, value) E.global.KlixUI.LOCK.TradeOpen = value; end,
					},
				},
			},
			fish = {
				order = 6,
				type = "group",
				name = T.GetSpell(7620),
				args = {
					easycast = {
						order = 1,
						name = L["Easy Cast"],
						desc = L["Allow to fish with double right-click."],
						type = "toggle",
						get = function(info) return E.private.KlixUI.professions.fishing.EasyCast end,
						set = function(info, value) E.private.KlixUI.professions.fishing.EasyCast = value; E:StaticPopup_Show("PRIVATE_RL") end,
					},
					fromMount = {
						order = 2,
						name = L["From Mount"],
						desc = L["Start fishing even if you are mounted."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.fishing.EasyCast end,
						get = function(info) return E.private.KlixUI.professions.fishing.FromMount end,
						set = function(info, value) E.private.KlixUI.professions.fishing.FromMount = value; end,
					},
					useLures = {
						order = 3,
						name = L["Apply Lures"],
						desc = L["Automatically apply lures."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.fishing.EasyCast end,
						get = function(info) return E.private.KlixUI.professions.fishing.UseLures end,
						set = function(info, value) E.private.KlixUI.professions.fishing.UseLures = value; end,
					},
					relureThreshold = {
						order = 4,
						type = 'range',
						name = L["Re-lure Threshold"],
						desc = L["Time after the previous attemp to apply a lure before the next attempt will occure."],
						min = 1, max = 15, step = 1,
						disabled = function() return not E.private.KlixUI.professions.fishing.EasyCast or not E.private.KlixUI.professions.fishing.UseLures end,
						get = function(info) return E.private.KlixUI.professions.fishing.relureThreshold end,
						set = function(info, value) E.private.KlixUI.professions.fishing.relureThreshold = value; end,
					},
					IgnorePole = {
						order = 5,
						name = L["Ignore Poles"],
						desc = L["If enabled will start fishing even if you don't have fishing pole equipped. Will not work if you have fish key set to \"None\"."],
						type = "toggle",
						disabled = function() return not E.private.KlixUI.professions.fishing.EasyCast or E.private.KlixUI.professions.fishing.CastButton == "None" end,
						get = function(info) return E.private.KlixUI.professions.fishing.IgnorePole end,
						set = function(info, value) E.private.KlixUI.professions.fishing.IgnorePole = value; end,
					},
					CastButton = {
						order = 6,
						name = L["Fish Key"],
						desc = L["Hold this button while clicking to allow fishing action."],
						type = "select",
						disabled = function() return not E.private.KlixUI.professions.fishing.EasyCast end,
						get = function(info) return E.private.KlixUI.professions.fishing.CastButton end,
						set = function(info, value) E.private.KlixUI.professions.fishing.CastButton = value; E:StaticPopup_Show("PRIVATE_RL") end,
						values = {
							["None"] = NONE,
							["Shift"] = SHIFT_KEY,
							["Alt"] = ALT_KEY,
							["Control"] = CTRL_KEY,
						},
					},
				},
			},
		},
	}
end
T.table_insert(KUI.Config, ProfessionsTable)